#version 450

// ----------------------------------------------------------------------------
// Input Variables
// ----------------------------------------------------------------------------

// Task 5.2: Add a uniform variable for eye(camera) position.
layout(location = 3) uniform vec3 eye_position;

// Task 5.2: Add a uniform variable for light's position and ambient+diffuse+specular terms.
layout(location = 4) uniform vec4 light_position;
layout(location = 5) uniform vec3 light_ambient_color;
layout(location = 6) uniform vec3 light_diffuse_color;
layout(location = 7) uniform vec3 light_specular_color;

// Task 5.2: Add a uniform variable for material's ambient+diffuse+specular+shininess terms.
layout(location = 8) uniform vec3  material_ambient_color;
layout(location = 9) uniform vec3  material_diffuse_color;
layout(location = 10) uniform vec3 material_specular_color;
layout(location = 11) uniform float material_shininess;


// The intpolated position from the vertex shader.
layout(location = 0) in vec3 fs_position;
// The intpolated normal from the vertex shader.
layout(location = 1) in vec3 fs_normal;
// The intpolated texture coordinates from the vertex shader.
layout(location = 2) in vec2 fs_texture_coordinate;

// ----------------------------------------------------------------------------
// Output Variables
// ----------------------------------------------------------------------------

// The final output color.
layout(location = 0) out vec4 final_color;

// ----------------------------------------------------------------------------
// Main Method
// ----------------------------------------------------------------------------
void main()
{
	// Task 5.2: Compute (N)ormal, (L)ight, (E)ye, (H)alf vectors.
	vec3 N = normalize(fs_normal);
	vec3 light_vector = light_position.xyz - fs_position * light_position.w;
	//if .w == 0 then directional_light (fs_position ignored)
	//if .w == 1 then positional_light
	vec3 L = normalize(light_vector);
	vec3 E = normalize(eye_position - fs_position);
	vec3 H = normalize(L+E);

	// Task 5.2: Compute dot products between N and L, N and H.
	float NdotL = max( dot(N,L), 0.0 );
	float NdotH = max( dot(N,H), 0.0 );

	// Task 5.2: Compute material values multiplied by light's values.
	vec3 ambient  = material_ambient_color  * light_ambient_color;
	vec3 diffuse  = material_diffuse_color  * light_diffuse_color;
	vec3 specular = material_specular_color * light_specular_color;

	// Task 5.2: Compute final color using Blinn�Phong shading model.
	vec3 color = ambient + NdotL*diffuse + pow(NdotH,material_shininess)*specular;

	// Task 5.2: Differentiate between directional and point light based on its .w value in position.
	float dist = light_position.w == 1 ? pow(length(light_vector),2) : 1.0;
	// Task 5.2: Use attenuation based on the light distance for point lights.
	final_color = vec4(color / dist, 1.0);
}
